/***************************************************************
**LED Watch Code
**
**This code describes the functions of the LED watch
**This watch tells time by a matrix of LEDs placed in an inner and outer
**rectangular patters, and a four LED column.
**The inner and outer LEDs are aligned like a standard 12 hr clock face
**
**To Read the Time 
**look at the placement of the outer green LED for the hour
**and the inner red LED for the minute (5 minute increments), then add
**the number of LEDs from the column to get the exact time
**
**To Set the Time:
**Press and hold the upper left button for a few seconds, until the LEDs flash
**Then use the right side buttons to increase or decrease the hour
**Once the hour is set, press the left button once to change the minute
**Use the left buttons again to increase or decrease the minute
**Wait a few seconds for the LEDs to stop flashing to enter default mode. 
*****************************************************************/

#include "MakerBotWatch.h"
MakerBotWatch watch;

/****************************************
**State Variables
**
**this controls the setting of the time
*****************************************/
byte buttonEvent = 0; //stores user input
long buttonTimer = millis(); //tells if button is being pressed
long buttonHoldTime = 1800; //amount of time to hold the button 
long timeSetTimeout = 8000; //the wait time before the watch goes to default

#define noButtonEvent      0
#define buttonEvent1       1
#define buttonEvent1Hold   2
#define buttonEvent2       3
#define buttonEvent2Hold   4
#define buttonEvent3       5
#define buttonEvent3Hold   6

byte state = 0;// the current state of the watch

//List of the various states of the watch
#define DEFAULT_STATE          0
#define EXIT_DEFAULT_STATE     1
#define ENTER_SET_TIME_STATE   2
#define SET_HOUR_STATE         3
#define INC_HOUR_STATE         4
#define DEC_HOUR_STATE         5
#define EXIT_HOUR_STATE        6
#define SET_MIN_STATE          7
#define INC_MIN_STATE          8
#define DEC_MIN_STATE          9
#define EXIT_MIN_STATE         10
#define ENTER_DEFAULT_STATE    11

/******************************************
**Clock variables
*******************************************/
int8_t hours;
int8_t minutes;

long lastClockTime=0, currentClockTime=0, lastTimeUpdated;//variable to keep the time going
const long oneMin = 60116; //length of one minute in ms (ADJUST AS NEEDED)

/*******************************************
**Input Variables
********************************************/
boolean PB1,PB2,PB3;//push button variables

boolean blinkVariable = 0; //blinks at a regular interval to show time is being set
byte blinkRate = 500; //the ms that it blinks 

/*******************************************
**Code that Runs once at Startup
********************************************/
void setup()
{
}

/**********************************************
**MAIN PROGRAM LOOP
***********************************************/
void loop()
{
  clockUpdate();//keeps the time current
  
  getUserInput();//check for button presses
  
  stateMachine();//decised what to do with user input
}

/*************************************************
**Checks the 3 buttons and updates the variable
**buttonEvent with the results
**************************************************/
void getUserInput(void)
{
  int oldButtonEvent = buttonEvent;
  
  //read the buttons
  PB1 = digitalRead(BUTTON1);
  PB2 = digitalRead(BUTTON2);
  PB3 = digitalRead(BUTTON3);
  
  //update output
  buttonEvent = noButtonEvent;
  if(!PB1)
    buttonEvent=buttonEvent1;
  if(!PB2)
    buttonEvent=buttonEvent2;
  if(!PB3)
    buttonEvent=buttonEvent3;
  
  //reset the timer if something changes
  if(oldButtonEvent != buttonEvent)
    buttonTimer = millis();
  
  //change the result if the button has been held for a set length of time
  if(millis()-buttonTimer > buttonHoldTime)
    {  
      if(!PB1)
        buttonEvent=buttonEvent1Hold;
      if(!PB2)
        buttonEvent=buttonEvent2Hold;
      if(!PB3)
        buttonEvent=buttonEvent3Hold;
    }
}
/*****************************************************
**This code decides what to do with the user input
******************************************************/
void stateMachine(void)
{
  //if nothind has happened for a while and the clock is not at default
  if(millis()-buttonTimer > timeSetTimeout && state != DEFAULT_STATE)
    state = DEFAULT_STATE;//put it back in default mode
    
  switch(state)
  {
  case DEFAULT_STATE:
    
    watch.setTime(hours, minutes);//set the display to show the time
    watch.showTime();//show the current time

      if(buttonEvent == buttonEvent3)//if button 3 is pushed
        state = EXIT_DEFAULT_STATE;//change state to decide what to do
 
    break;
    
    case EXIT_DEFAULT_STATE:
    watch.setTime(hours, minutes);
    watch.showTime();
    if(buttonEvent == noButtonEvent)//if all buttons are releases
      state = DEFAULT_STATE;//go back to the default state
    if(buttonEvent == buttonEvent3Hold)//if button 3 is held
      state = ENTER_SET_TIME_STATE;//set the time
    break;
    
    case ENTER_SET_TIME_STATE:
      if(buttonEvent == noButtonEvent)//wait for the user to release button
        state = SET_HOUR_STATE;//change state to set the hour
        break;
        
    case SET_HOUR_STATE:
      updateBlink();//update the flashing variable for display
      watch.setTime(hours, minutes);
      if(blinkVariable==HIGH)
        watch.showTime();
      if(buttonEvent == buttonEvent2)//if button 1 is pushed
        state = INC_HOUR_STATE;//increase the hour
      if(buttonEvent == buttonEvent1)//if button 2 is pushed
        state = DEC_HOUR_STATE;//decrease the hour
      if(buttonEvent == buttonEvent3)//if button3 is pushed
        state = EXIT_HOUR_STATE;//wait for user
      break;
      
  case INC_HOUR_STATE:
    watch.setTime(hours, minutes);
    watch.showTime();
    if(buttonEvent == noButtonEvent)//wait for the user to release the button
    {
      hours = (hours+1)%12;//increase the hour by one
      
      state = SET_HOUR_STATE;//and return to change the hour again
    }
    break;
    
   case DEC_HOUR_STATE:
    watch.setTime(hours, minutes);
    watch.showTime();
    if(buttonEvent == noButtonEvent)//wait for the user to release the button
    {
      hours = (hours-1);//decrease the hour by one
      if(hours <0)
      {
        hours=11;
      }
      state = SET_HOUR_STATE;//and return to change the hour again
    }
    break;
    
   case EXIT_HOUR_STATE:
    watch.setTime(hours, minutes);
    watch.showTime();
    if(buttonEvent == noButtonEvent)//wait for the user to release the button
      state = SET_MIN_STATE;//and change state to set the minutes
   
    break;
    
   case SET_MIN_STATE:
    updateBlink();
    watch.setTime(hours, minutes);
    if(blinkVariable==HIGH)
      watch.showTime();
    if(buttonEvent == buttonEvent2)//if button 1 is pushed
      state = INC_MIN_STATE;//increase the minutes
    if(buttonEvent == buttonEvent1)//if button 2 is pushed
      state = DEC_MIN_STATE;//decrease the minutes
    if(buttonEvent == buttonEvent3)//if button 3 is pushed
      state = EXIT_MIN_STATE;//wait for the user to release in this state and decide what to do
    break;
    
   case INC_MIN_STATE:
    watch.setTime(hours, minutes);
    watch.showTime();
    if(buttonEvent == noButtonEvent)//when all buttons are released
    {
      minutes = (minutes+1)%60;//increase the minutes by one
      state = SET_MIN_STATE;//and go back to change the minutes again
    }
    break;
    
   case DEC_MIN_STATE:
    watch.setTime(hours, minutes);
    watch.showTime();
    if(buttonEvent == noButtonEvent)//when all buttons are released
    {
      minutes = (minutes-1);//decrease the minutes by one
      if(minutes < 0)
        minutes=59;
      state = SET_MIN_STATE;//and go back to change the minutes again
    }
    break;
    
   case EXIT_MIN_STATE:
    watch.setTime(hours, minutes);
    watch.showTime();
    if(buttonEvent == noButtonEvent)//when the button is released
      state = SET_HOUR_STATE;//change state to set the hour

    break;
    
   case ENTER_DEFAULT_STATE:
    if(buttonEvent == noButtonEvent)//wait until the buttons are released
      state = DEFAULT_STATE;//and go back to the default state
    break;
    
   default://if something unexpected happens
    state = DEFAULT_STATE;//go back to the default state (time display)
    break;
  }
}

void updateBlink(void)
{
  if(millis()%blinkRate>=blinkRate/2)
    blinkVariable=1;
  else
    blinkVariable=0;
}

void clockUpdate()
{
  lastClockTime=currentClockTime;//for when millis wraps around
  currentClockTime=millis();

  if(currentClockTime<lastTimeUpdated)//catch when the timer wraps around
      lastTimeUpdated-=lastClockTime;

  if(currentClockTime-oneMin>=lastTimeUpdated)//update the time
  {
    lastTimeUpdated+=oneMin;//update the counter

    minutes=(minutes+1)%60;//get the new time
    if (minutes==0)
    {
      hours=(hours+1)%12;
    }
      }
}


